@@ -4,7 +4,7 @@ package Perl::Dist::Strawberry;
 
 =head1 NAME
 
-Perl::Dist::Strawberry - Strawberry Perl for win32
+Perl::Dist::Strawberry - Strawberry Perl for Win32
 
 =head1 DESCRIPTION
 
@@ -128,8 +128,9 @@ use URI::file                        qw();
 use File::ShareDir                   qw();
 use Perl::Dist::WiX::Util::Machine   qw();
 use File::List::Object               qw();
+use Path::Class::Dir                 qw();
 
-our $VERSION = '2.02_05';
+our $VERSION = '2.10_10';
 $VERSION =~ s/_//ms;
 
 #####################################################################
@@ -142,13 +143,13 @@ $VERSION =~ s/_//ms;
   Perl::Dist::Strawberry->default_machine(...)->run();
   
 The C<default_machine> class method is used to setup the most common
-machine for building Strawberry Perl.
+'machine' for building Strawberry Perl.
 
 The machine provided creates a standard 5.8.9 distribution (.zip and .msi),
 a standard 5.10.1 distribution (.zip and .msi) and a Portable-enabled 5.10.1 
 distribution (.zip only).
 
-Returns a L<Perl::Dist::WiX::Util::Machine> object.
+Returns a L<Perl::Dist::WiX::Util::Machine|Perl::Dist::WiX::Util::Machine> object.
 
 =cut
 
@@ -164,21 +165,30 @@ sub default_machine {
 	# Set the different versions
 	$machine->add_dimension('version');
 	$machine->add_option('version',
-		perl_version => '589',
-	    build_number => 5,
+		perl_version => '5101',
+		build_number => 3,
 	);
 	$machine->add_option('version',
 		perl_version => '5101',
+		build_number => 3,
+		image_dir    => 'D:\strawberry',
+		msi          => 1,
+		zip          => 0,
 	);
 	$machine->add_option('version',
-		perl_version => '5101',
-		image_dir => 'D:\strawberry',
-		msi       => 1,
-		zip       => 0,
-	);
-	$machine->add_option('version',
-		perl_version => '5101',
+		perl_version => '5121',
+		build_number => 0,
 		portable     => 1,
+		gcc_version  => 4,
+		download_dir => 'C:\tmp\dl-gcc4',
+	);
+	$machine->add_option('version',
+		perl_version       => '5121',
+		build_number       => 0,
+		relocatable        => 1,
+		use_dll_relocation => 1,
+		gcc_version        => 4,
+		download_dir       => 'C:\tmp\dl-gcc4',
 	);
 
 	return $machine;
@@ -193,10 +203,10 @@ sub default_machine {
 
 # Apply default paths
 sub new {
-	my $dist_dir = File::ShareDir::dist_dir('Perl-Dist-Strawberry');
+	my $dist_dir = Path::Class::Dir->new(File::ShareDir::dist_dir('Perl-Dist-Strawberry'));
 	my $class = shift;
 	
-	if ($Perl::Dist::WiX::VERSION < '1.102102') {
+	if ($Perl::Dist::WiX::VERSION < '1.200100') {
 		PDWiX->throw('Perl::Dist::WiX version is not high enough.')
 	}
 
@@ -208,19 +218,21 @@ sub new {
 		image_dir            => 'C:\strawberry',
 
 		# Perl version
-		perl_version         => '5101',
+		perl_version         => '5121',
 		
 		# Program version.
-		build_number         => 2,
+		build_number         => 0,
 		beta_number          => 1,
 		
 		# New options for msi building...
-		msi_license_file     => catfile($dist_dir, 'License-short.rtf'),
+		msi_license_file     => $dist_dir->file('License-short.rtf'),
 		msi_product_icon     => catfile(File::ShareDir::dist_dir('Perl-Dist-WiX'), 'win32.ico'),
 		msi_help_url         => 'http://www.strawberryperl.com/support.html',
-		msi_banner_top       => catfile($dist_dir, 'StrawberryBanner.bmp'),
-		msi_banner_side      => catfile($dist_dir, 'StrawberryDialog.bmp'),
-
+		msi_banner_top       => $dist_dir->file('StrawberryBanner.bmp'),
+		msi_banner_side      => $dist_dir->file('StrawberryDialog.bmp'),
+		msi_exit_text        => <<'EOT',
+Before you start using Strawberry Perl, read the Release Notes and the README file.  These are both available from the start menu under "Strawberry Perl".
+EOT
 		# Set e-mail to something Strawberry-specific.
 		perl_config_cf_email => 'win32-vanilla@perl.org',
 
@@ -243,7 +255,6 @@ sub new {
 			'install_strawberry_modules_4',
 			'install_strawberry_modules_5',
 			'install_strawberry_files',
-			'add_forgotten_files',
 			'install_relocatable',
 			'regenerate_fragments',
 			'find_relocatable_fields',
@@ -315,34 +326,50 @@ sub install_strawberry_c_libraries {
 	my $self = shift;
 
 	# XML Libraries
-	$self->install_zlib();
-	$self->install_libiconv();
-	$self->install_libxml();
-	$self->install_expat();
-	$self->install_libxslt();
+	$self->install_librarypacks(qw{
+		zlib
+		libiconv
+		libxml2
+		libexpat
+		libxslt
+	});
 
 	# Math Libraries
-	$self->install_gmp();
+	$self->install_librarypacks(qw{
+		gmp
+		mpc
+		mpfr
+	});
 
 	# Graphics libraries
-	$self->install_libjpeg();
-	$self->install_libgif();
-	$self->install_libtiff();
-	$self->install_libpng();
-	$self->install_libgd();
-	$self->install_libfreetype();
-	$self->install_libxpm();	
+	$self->install_librarypacks(qw{
+		libjpeg
+		libgif
+		libtiff
+		libpng
+		libgd
+		libfreetype
+		libxpm
+		freeglut
+	});	
 	
 	# Database Libraries
-	$self->install_libdb();
-	$self->install_libgdbm();
- 	if (32 == $self->bits()) {
-		$self->install_libpostgresql();
-	}
+	$self->install_librarypacks(qw{
+		libdb
+		libgdbm
+		libpostgresql
+		libmysql
+	});
+
+	# Extra compression libraries
+	$self->install_librarypack('libxz');
 
 	# Crypto libraries
-	$self->install_libopenssl();
-	
+	$self->install_librarypacks(qw{
+		libopenssl
+		libssh2
+	});
+
 	return 1;
 }
 
@@ -387,12 +414,17 @@ sub install_perl_bin {
 	my $self   = shift;
 	my %params = @_;
 	my $patch  = delete($params{patch}) || [];
+	
+	# Patch this file so GDBM_File will build.
+	my @files_to_patch = ('win32/FindExt.pm');
+
+	# If we aren't a git checkout or a 5.12 version, patch up GDBM_File.
+	if ( $self->perl_version() !~ m/ \A512 | \Agit\z /msx) {
+		push @files_to_patch, qw(ext/GDBM_File/GDBM_File.xs ext/GDBM_File/GDBM_File.pm);
+	}
+
 	return $self->SUPER::install_perl_bin(
-		patch => [ qw{
-			win32/FindExt.pm
-			ext/GDBM_File/GDBM_File.xs
-			ext/GDBM_File/GDBM_File.pm
-		}, @$patch ],
+		patch => [ @files_to_patch, @$patch ],
 		%params,
 	);
 }
@@ -401,7 +433,7 @@ sub install_strawberry_modules_1 {
 	my $self = shift;
 
 	# Install LWP::Online so our custom minicpan code works
-	if ($self->portable()) {
+	if ($self->portable() && (12 < $self->perl_major_version()) ) {
 		$self->install_distribution(
 			name     => 'ADAMK/LWP-Online-1.07.tar.gz',
 			mod_name => 'LWP::Online',
@@ -421,7 +453,6 @@ sub install_strawberry_modules_1 {
 		File::Remove
 		Win32::File::Object
 		Parse::Binary
-		Win32::Exe
 		Win32::EventLog
 	} );
 	$self->install_modules('Win32::API') if not 64 == $self->bits();
@@ -433,14 +464,14 @@ sub install_strawberry_modules_1 {
 	} );
 	
 	# XML Modules
-	if ($self->portable()) {
+	if ($self->portable() && (12 < $self->perl_major_version()) ) {
 		$self->install_distribution(
 			name             => 'MSERGEANT/XML-Parser-2.36.tar.gz',
 			mod_name         => 'XML::Parser',
 			makefilepl_param => [
 				'INSTALLDIRS=site',
-				'EXPATLIBPATH=' . $self->_dir(qw{ c lib     }),
-				'EXPATINCPATH=' . $self->_dir(qw{ c include }),
+				'EXPATLIBPATH=' . $self->dir(qw{ c lib     }),
+				'EXPATINCPATH=' . $self->dir(qw{ c include }),
 			],
 		);
 	} else {
@@ -449,8 +480,8 @@ sub install_strawberry_modules_1 {
 			mod_name         => 'XML::Parser',
 			makefilepl_param => [
 				'INSTALLDIRS=vendor',
-				'EXPATLIBPATH=' . $self->_dir(qw{ c lib     }),
-				'EXPATINCPATH=' . $self->_dir(qw{ c include }),
+				'EXPATLIBPATH=' . $self->dir(qw{ c lib     }),
+				'EXPATINCPATH=' . $self->dir(qw{ c include }),
 			],
 		);
 	}
@@ -462,12 +493,18 @@ sub install_strawberry_modules_1 {
 		XML::LibXSLT
 	} );
 	
-	unless ($self->portable()) {
+	unless ($self->portable() && (12 < $self->perl_major_version()) ) {
 		# Insert ParserDetails.ini
 		my $ini_file = catfile($self->image_dir(), qw(perl vendor lib XML SAX ParserDetails.ini));
 		$self->add_to_fragment('XML_SAX', [ $ini_file ]);
 	}
 	
+	# Apparently Win32::Exe now requires XML::Simple and XML::Parser
+	$self->install_modules( qw{
+		XML::Simple
+		Win32::Exe
+	} );
+
 	return 1;
 }
 
@@ -490,34 +527,22 @@ sub install_strawberry_modules_2 {
 		Test::NoWarnings
 		Test::Deep
 		IO::Stringy
+		Test::Exception
 	} );
-	
-	if ($self->portable()) {
-		$self->install_distribution(
-			name             => 'RKINYON/DBM-Deep-1.0013.tar.gz',
-			mod_name         => 'DBM::Deep',
-			makefilepl_param => ['INSTALLDIRS=site'],
-			buildpl_param    => ['--installdirs', 'site'],
-			force            => 1,
-		);
-	} else {
-		$self->install_distribution(
-			name             => 'RKINYON/DBM-Deep-1.0013.tar.gz',
-			mod_name         => 'DBM::Deep',
-			makefilepl_param => ['INSTALLDIRS=vendor'],
-			buildpl_param    => ['--installdirs', 'vendor'],
-			force            => 1,
-		);
-	}
-	
+	$self->install_module(
+		name => 'DBM::Deep',
+		force => 1, # RT#56512. (missing t\lib directory) Other tests pass.
+	);
 	$self->install_modules( qw{
 		YAML::Tiny
 		PAR
 		PAR::Repository::Query
 		PAR::Repository::Client
 	} );
-	$self->install_ppm();
-
+	if (32 == $self->bits()) {
+		$self->install_ppm();
+	}
+	
 	my $cpan_sources = catdir($self->image_dir, 'cpan', 'sources');
 	unless (-d $cpan_sources) {
 		require File::Path;
@@ -567,29 +592,31 @@ sub install_strawberry_modules_3 {
 	); 
 	$self->install_modules( qw{
 		BerkeleyDB
+		Win32::OLE
 		DBD::ODBC
+		DBD::ADO
 	} );
 
-	if (3 == $self->gcc_version()) {
-		$self->install_dbd_mysql();
-		
-		my $install_location = $self->portable() ? q{perl\site\lib} : q{perl\vendor\lib};	
-		my $mysql_url = $self->get_library_file('mysqllib');
-		
-		my $filelist = $self->install_binary(
-			name       => 'db_libraries',
-			url        => $self->_binary_url($mysql_url),
-			install_to => $install_location
+	if ($self->portable() && (12 < $self->perl_major_version()) ) {
+		$self->install_distribution(
+			name     => 'CAPTTOFU/DBD-mysql-4.014.tar.gz',
+			mod_name => 'DBD::mysql',
+			force    => 1,
+			makefilepl_param => ['INSTALLDIRS=site', '--mysql_config=mysql_config'],
 		);
-		$self->insert_fragment( 'db_libraries', $filelist );
-	}
-	
-	if (32 == $self->bits()) {
-		$self->install_module(
-			name  => 'DBD::Pg',
-			force => 1,
+	} else {
+		$self->install_distribution(
+			name     => 'CAPTTOFU/DBD-mysql-4.014.tar.gz',
+			mod_name => 'DBD::mysql',
+			force    => 1,
+			makefilepl_param => ['INSTALLDIRS=vendor', '--mysql_config=mysql_config'],
 		);
 	}
+			
+	$self->install_module(
+		name  => 'DBD::Pg',
+		force => 1,
+	);
 	
 	# JSON and local library installation
 	$self->install_modules( qw{
@@ -600,7 +627,6 @@ sub install_strawberry_modules_3 {
 	} );	
 
 	# Graphics module installation.
-	$self->_remake_path(catdir($self->image_dir(), qw(cpan build))); 
 	$self->install_module( name => 'Imager' );
 	$self->install_module( name => 'GD' );
 	
@@ -609,42 +635,44 @@ sub install_strawberry_modules_3 {
 
 sub install_strawberry_modules_4 {
 	my $self = shift;
-
-	# This is due to RT #52408.
-	return 1 if 64 == $self->bits();
 	
+#	if ($self->portable() && (12 < $self->perl_major_version()) ) {
+#		$self->install_distribution( 
+#			mod_name => 'Crypt::OpenSSL::Random',
+#			name     => 'IROBERTS/Crypt-OpenSSL-Random-0.04.tar.gz',
+#			makefilepl_param => [
+#				'LIBS="-lssl32 -leay32"' ,
+#			],
+#		);
+#	} else {
+#		$self->install_distribution( 
+#			mod_name => 'Crypt::OpenSSL::Random',
+#			name     => 'IROBERTS/Crypt-OpenSSL-Random-0.04.tar.gz',
+#			makefilepl_param => [
+#				'INSTALLDIRS=vendor', 'LIBS="-lssl32 -leay32"' ,
+#			],
+#		);
+#	}
+
 	# Required for Net::SSLeay.
-	local $ENV{'OPENSSL_PREFIX'} = catdir($self->image_dir, 'c');
+	local $ENV{'OPENSSL_PREFIX'} = catdir($self->image_dir(), 'c');
 	# This is required for IO::Socket::SSL.
 	local $ENV{'SKIP_RNG_TEST'} = 1;
 
-	# We have to tell the Makefile.PL where the OpenSSL 
-	# libraries are by passing a parameter for Crypt::SSLeay.
-	if ($self->portable()) {
-		$self->install_distribution( 
-			mod_name => 'Crypt::SSLeay',
-			name     => 'DLAND/Crypt-SSLeay-0.57.tar.gz',
-			makefilepl_param => [
-				'--lib', $ENV{'OPENSSL_PREFIX'} ,
-			],
-		);
-	} else {
-		$self->install_distribution( 
-			mod_name => 'Crypt::SSLeay',
-			name     => 'DLAND/Crypt-SSLeay-0.57.tar.gz',
-			makefilepl_param => [
-				'INSTALLDIRS=vendor', '--lib', $ENV{'OPENSSL_PREFIX'} ,
-			],
-		);
-	}
-	
+	# Crypt::SSLeay has been distropref'd to use the same environment
+	# variable that Net::SSLeay uses in order to make building easier.
 	$self->install_modules( qw{
-		Net::SSLeay
+		Crypt::SSLeay
 		Digest::HMAC
-		IO::Socket::SSL
-		Net::SMTP::TLS
 	});
 
+	# Net::SSLeay crashes at present on 64-bit during testing.
+	$self->install_modules( qw{	
+		Net::SSLeay
+		IO::Socket::SSL
+		Net::SMTP::TLS
+	}) if 32 == $self->bits();
+	
 	# The rest of the Net::SSH::Perl toolchain.
 	$self->install_module(
 		name  => 'Data::Random',
@@ -653,7 +681,12 @@ sub install_strawberry_modules_4 {
 	$self->install_modules( qw{
 		Math::GMP
 		Data::Buffer
+	});
+	# Check why this one isn't working.
+	$self->install_modules( qw{
 		Crypt::DSA
+	}) if 32 == $self->bits();
+	$self->install_modules( qw{
 		Class::ErrorHandler
 		Convert::ASN1
 		Crypt::CBC
@@ -670,15 +703,31 @@ sub install_strawberry_modules_4 {
 		Crypt::Blowfish
 		Tie::EncryptedHash
 		Class::Loader
+	});
+	# Requires Math::Pari.
+	$self->install_modules( qw{
 		Crypt::Random
+	}) if 32 == $self->bits();
+	$self->install_modules( qw{
 		Convert::ASCII::Armour
 		Digest::MD2
 		Sort::Versions
+	});
+	# These two require Crypt::Random. See above.
+	$self->install_modules( qw{
 		Crypt::Primes
 		Crypt::RSA
+	}) if 32 == $self->bits();
+	$self->install_modules( qw{
 		Digest::BubbleBabble
+	});
+	# Does not build 64-bit yet.
+	$self->install_modules( qw{
 		Crypt::IDEA
+	}) if 32 == $self->bits();
+	$self->install_modules( qw{
 		String::CRC32
+		Net::SSH2
 	});
 
 	# Since	Net::SSH::Perl does not work under Win32 yet, it
@@ -692,12 +741,20 @@ sub install_strawberry_modules_4 {
 		Crypt::CAST5_PP
 		Crypt::RIPEMD160
 		Crypt::Twofish
-		Test::Exception
+	});
+	# Requires Crypt::DSA, Crypt::IDEA, 
+	# Crypt::RSA, and Math::Pari.
+	$self->install_modules( qw{
 		Crypt::OpenPGP
+	}) if 32 == $self->bits();
+	$self->install_modules( qw{
 		Algorithm::Diff
 		Text::Diff
-		Module::Signature
 	});
+	# Requires Crypt::OpenPGP - see above.
+	$self->install_modules( qw{
+		Module::Signature
+	}) if 32 == $self->bits();
 	
 	return 1;
 }
@@ -709,14 +766,30 @@ sub install_strawberry_modules_5 {
 	$self->install_modules( qw{
 		File::Slurp
 		Task::Weaken
-	});
-	# This is fixed by a distropref.
-	$self->install_modules( qw{
 		SOAP::Lite
+		Class::Inspector
+		File::ShareDir
+		Alien::Tidyp
+	});
+	
+	# For the local-lib script.
+	$self->install_modules( qw{
+		IO::Interactive
+		App::local::lib::Win32Helper
 	});
 
+	# Additional compression modules
+	$self->install_module( name => 'Compress::Raw::Lzma' );
+	$self->install_module( name => 'IO::Compress::Lzma', force => 1 );
+	
+	# Additional math modules.
+	$self->install_modules( qw{
+		Math::MPFR
+		Math::MPC
+	});
+	
 	# Clear things out.
-	$self->_remake_path(catdir($self->image_dir(), qw(cpan build))); 
+	$self->remake_path($self->dir(qw(cpan build))); 
 
 	return 1;
 }
@@ -727,25 +800,31 @@ sub install_strawberry_files {
 	## Now let's copy individual files in.
 	
 	# Copy the module-version script in, and use the runperl.bat trick on it.
-	$self->_copy(catfile($self->dist_dir(), 'module-version'), catdir($self->image_dir(), qw(perl bin)));
-	$self->_copy(catfile($self->image_dir(), qw(perl bin runperl.bat)), catfile($self->image_dir(), qw(perl bin module-version.bat)));
+	$self->copy_file(
+		catfile($self->dist_dir(), 'module-version'), 
+		$self->file(qw(perl bin module-version))
+	);
+	$self->copy_file(
+		$self->file(qw(perl bin runperl.bat)), 
+		$self->file(qw(perl bin module-version.bat))
+	);
 	
 	# Make sure it gets installed.
 	$self->insert_fragment('module_version',
 		File::List::Object->new()->add_files(
-			catfile($self->image_dir(), qw(perl bin module-version)),
-			catfile($self->image_dir(), qw(perl bin module-version.bat)),				
+			$self->file(qw(perl bin module-version)),
+			$self->file(qw(perl bin module-version.bat)),				
 		),
 	);
 
 	if ($self->relocatable()) {
 		# Copy the relocation information in.
-		$self->patch_file('strawberry-merge-module.reloc.txt', $self->image_dir());
+		$self->make_relocation_file('strawberry-merge-module.reloc.txt');
 		
 		# Make sure it gets installed.
 		$self->insert_fragment('relocation_info',
 			File::List::Object->new()->add_file(
-				catfile($self->image_dir(), 'strawberry-merge-module.reloc.txt'),				
+				$self->file('strawberry-merge-module.reloc.txt'),				
 			),
 		);
 	}
@@ -758,6 +837,10 @@ sub install_strawberry_files {
 #####################################################################
 # Customisations to Windows assets
 
+sub _dist_file {
+	return File::ShareDir::dist_file('Perl-Dist-Strawberry', @_);
+}
+
 sub install_strawberry_extras {
 	my $self = shift;
 
@@ -766,39 +849,59 @@ sub install_strawberry_extras {
 	# Links to the Strawberry Perl website.
 	# Don't include this for non-Strawberry sub-classes
 	if ( ref($self) eq 'Perl::Dist::Strawberry' ) {
+		$self->patch_file( 'README.txt' => $self->image_dir(), { dist => $self } );
 		if (not $self->portable()) {
 			$self->install_launcher(
 				name => 'Check installed versions of modules',
 				bin  => 'module-version',
 			);
+			$self->install_launcher(
+				name => 'Create local library areas',
+				bin  => 'llw32helper',
+			);
 			$self->install_website(
 				name       => 'Strawberry Perl Website',
 				url        => $self->strawberry_url(),
-				icon_file  => catfile($dist_dir, 'strawberry.ico')
+				icon_file  => _dist_file('strawberry.ico')
 			);
 			$self->install_website(
-				name       => 'Strawberry Perl Release Notes',
-				url        => $self->strawberry_release_notes_url(),
-				icon_file  => catfile($dist_dir, 'strawberry.ico')
+				name         => 'Strawberry Perl Release Notes',
+				url          => $self->strawberry_release_notes_url(),
+				icon_file    => _dist_file('strawberry.ico'),
+				directory_id => 'App_Menu',
+			);
+			$self->install_website(
+				name         => 'learn.perl.org (tutorials, links)',
+				url          => 'http://learn.perl.org/',
+				icon_file    => _dist_file('perlhelp.ico'),
+			);
+			$self->install_website(
+				name         => 'Beginning Perl (online book)',
+				url          => 'http://learn.perl.org/books/beginning-perl/',
+				icon_file    => _dist_file('perlhelp.ico'),
 			);
 			# Link to IRC.
 			$self->install_website(
 				name       => 'Live Support',
 				url        => 'http://widget.mibbit.com/?server=irc.perl.org&channel=%23win32',
-				icon_file  => catfile($dist_dir, 'onion.ico')
+				icon_file  => _dist_file('onion.ico')
 			);
+#			$self->add_icon(
+#				name         => 'Strawberry Perl README',
+#				directory_id => 'App_Menu',
+#				filename     => $self->image_dir()->file('README.txt')->stringify(),
+#			);
 		}
-		$self->patch_file( 'README.txt' => $self->image_dir(), { dist => $self } );
 	}
 
 	my $license_file_from = catfile($dist_dir, 'License.rtf');
 	my $license_file_to = catfile($self->license_dir(), 'License.rtf');
-	my $readme_file = catfile($self->image_dir(), 'README.txt');
+	my $readme_file = $self->file('README.txt');
 
-	my $onion_ico_file = catfile($self->image_dir(), qw(win32 onion.ico));
-	my $strawberry_ico_file = catfile($self->image_dir(), qw(win32 strawberry.ico));
+	my $onion_ico_file = $self->file(qw(win32 onion.ico));
+	my $strawberry_ico_file = $self->file(qw(win32 strawberry.ico));
 	
-	$self->_copy($license_file_from, $license_file_to);	
+	$self->copy_file($license_file_from, $license_file_to);	
 	if (not $self->portable()) {
 		$self->add_to_fragment( 'Win32Extras',
 			[ $license_file_to, $readme_file, $onion_ico_file, $strawberry_ico_file ] );
@@ -806,12 +909,12 @@ sub install_strawberry_extras {
 
 	if ($self->relocatable()) {
 		# Copy the relocation information in.
-		$self->patch_file('strawberry-ui.reloc.txt', $self->image_dir());
+		$self->make_relocation_file('strawberry-ui.reloc.txt', 'strawberry-merge-module.reloc.txt');
 		
 		# Make sure it gets installed.
 		$self->insert_fragment('relocation_ui_info',
 			File::List::Object->new()->add_file(
-				catfile($self->image_dir(), 'strawberry-ui.reloc.txt'),				
+				$self->file('strawberry-ui.reloc.txt'),				
 			),
 		);
 	}
@@ -844,15 +947,50 @@ sub strawberry_release_notes_url {
 sub msi_relocation_commandline_files {
 	my $self = shift;
 	
-	return('relocation_ui_info', catfile($self->image_dir(), 'strawberry-ui.reloc.txt'));
+	return('relocation_ui_info', $self->file('strawberry-ui.reloc.txt'));
 }
 
 sub msm_relocation_commandline_files {
 	my $self = shift;
 	
-	return('relocation_info', catfile($self->image_dir(), 'strawberry-merge-module.reloc.txt'));
+	return('relocation_info', $self->file('strawberry-merge-module.reloc.txt'));
 }
 
+sub msi_relocation_idlist {
+	my $self = shift;
+
+	my $answer;
+	my %files = $self->msi_relocation_commandline_files();
+
+	my ( $fragment, $file, $id );
+	while ( ( $fragment, $file ) = each %files ) {
+		$id = $self->get_fragment_object($fragment)->find_file_id($file);
+		PDWiX->throw("Could not find file $file in fragment $fragment\n")
+		  if not defined $id;
+		$answer .= "[#$id]";
+	}
+
+	return $answer;
+} ## end sub msi_relocation_commandline
+
+sub msm_relocation_idlist {
+	my $self = shift;
+
+	my $answer;
+	my %files = $self->msm_relocation_commandline_files();
+
+	my ( $fragment, $file, $id );
+	while ( ( $fragment, $file ) = each %files ) {
+		$id = $self->get_fragment_object($fragment)->find_file_id($file);
+		PDWiX->throw("Could not find file $file in fragment $fragment\n")
+		  if not defined $id;
+		$answer .= "[#$id]";
+	}
+
+	return $answer;
+} ## end sub msi_relocation_commandline
+
+
 1;
 
 =pod
