@@ -36,7 +36,7 @@ use warnings;
 use File::Spec::Functions qw( catfile catdir );
 use Readonly;
 
-our $VERSION = '2.02_05';
+our $VERSION = '2.10_10';
 $VERSION =~ s/_//ms;
 
 Readonly my %LIBRARIES_S => {
@@ -47,12 +47,14 @@ Readonly my %LIBRARIES_S => {
 		'mysql5101'     => 'DBD-mysql-4.012-MSWin32-x86-multi-thread-5.10.0.par',
 		'mysql5115'     => undef,
 		'mysql5120'     => undef,
+		'mysql5121'     => undef,
 		'mysqllib'      => '32bit-gcc3/MySQLLibraries-20100121.zip',
 		'pari589'       => 'Math-Pari-2.010801-MSWin32-x86-multi-thread-5.8.9.par',
 		'pari5100'      => '32bit-gcc3/Math-Pari-2.01080603-MSWin32-x86-multi-thread-5.10.1.par',
 		'pari5101'      => '32bit-gcc3/Math-Pari-2.01080603-MSWin32-x86-multi-thread-5.10.1.par',
 		'pari5115'      => '32bit-gcc3/Math-Pari-2.01080603-MSWin32-x86-multi-thread-5.11.5.par',
 		'pari5120'      => undef,
+		'pari5121'      => undef,
 		'zlib'          => '32bit-gcc4/zlib-1.2.3-bin_20091126.zip',
 		'libiconv'      => '32bit-gcc4/libiconv-1.13.1-bin_20091126.zip',
 		'libxml2'       => '32bit-gcc4/libxml2-2.7.3-bin_20091126.zip',
@@ -70,6 +72,12 @@ Readonly my %LIBRARIES_S => {
 		'libdb'         => '32bit-gcc4/db-4.8.24-bin_20091126.zip',
 		'libgdbm'       => '32bit-gcc4/gdbm-1.8.3-bin_20100112.zip',
 		'libxpm'        => '32bit-gcc4/libXpm-3.5.8-bin_20091126.zip',
+		'libxz'         => '32bit-gcc4/liblzma-xz-4.999.9beta-bin_20100308.zip',
+		'mpc'           => '32bit-gcc4/mpc-0.8.1-bin_20100306.zip',
+		'mpfr'          => '32bit-gcc4/mpfr-2.4.2-bin_20100306.zip',
+		'libmysql'      => '32bit-gcc4/mysql-5.1.44-bin_20100304.zip',
+		'freeglut'      => '32bit-gcc4/freeglut-2.6.0-bin_20100213.zip',
+		'libssh2'       => '32bit-gcc4/libssh2-1.2.5-bin_20100520.zip',
 	},
 	'32bit-gcc4' => {
 		'patch'         => '32bit-gcc4/patch-2.5.9-7-bin_20100110_20100303.zip',
@@ -78,12 +86,14 @@ Readonly my %LIBRARIES_S => {
 		'mysql5101'     => undef,
 		'mysql5115'     => undef,
 		'mysql5120'     => undef,
-		'mysqllib'      => '32bit-gcc3/MySQLLibraries-20100121.zip',
+		'mysql5121'     => undef,
+		'mysqllib'      => '32bit-gcc4/mysql-5.1.44-bin_20100304.zip',
 		'pari589'       => undef,
 		'pari5100'      => undef,
 		'pari5101'      => undef,
 		'pari5115'      => '32bit-gcc4/Math-Pari-2.01080604-MSWin32-x86-multi-thread-5.11.5.par',
 		'pari5120'      => '32bit-gcc4/Math-Pari-2.01080604-MSWin32-x86-multi-thread-5.12.0.par',
+		'pari5121'      => '32bit-gcc4/Math-Pari-2.01080604-MSWin32-x86-multi-thread-5.12.0.par',
 		'zlib'          => '32bit-gcc4/zlib-1.2.3-bin_20091126.zip',
 		'libiconv'      => '32bit-gcc4/libiconv-1.13.1-bin_20091126.zip',
 		'libxml2'       => '32bit-gcc4/libxml2-2.7.3-bin_20091126.zip',
@@ -101,6 +111,12 @@ Readonly my %LIBRARIES_S => {
 		'libdb'         => '32bit-gcc4/db-4.8.24-bin_20091126.zip',
 		'libgdbm'       => '32bit-gcc4/gdbm-1.8.3-bin_20100112.zip',
 		'libxpm'        => '32bit-gcc4/libXpm-3.5.8-bin_20091126.zip',
+		'libxz'         => '32bit-gcc4/liblzma-xz-4.999.9beta-bin_20100308.zip',
+		'mpc'           => '32bit-gcc4/mpc-0.8.1-bin_20100306.zip',
+		'mpfr'          => '32bit-gcc4/mpfr-2.4.2-bin_20100306.zip',
+		'libmysql'      => '32bit-gcc4/mysql-5.1.44-bin_20100304.zip',
+		'freeglut'      => '32bit-gcc4/freeglut-2.6.0-bin_20100213.zip',
+		'libssh2'       => '32bit-gcc4/libssh2-1.2.5-bin_20100520.zip',
 	},
 	'64bit-gcc4' => {
 		'patch'         => '64bit-gcc4/patch-2.5.9-7-bin_20100110_20100303.zip',
@@ -109,12 +125,14 @@ Readonly my %LIBRARIES_S => {
 		'mysql5101'     => undef,
 		'mysql5115'     => undef,
 		'mysql5120'     => undef,
+		'mysql5121'     => undef,
 		'mysqllib'      => undef,
 		'pari589'       => undef,
 		'pari5100'      => undef,
 		'pari5101'      => undef,
 		'pari5115'      => undef,
 		'pari5120'      => undef,
+		'pari5121'      => undef,
 		'zlib'          => '64bit-gcc4/zlib-1.2.3-bin_20100110.zip',
 		'libiconv'      => '64bit-gcc4/libiconv-1.13.1-bin_20100110.zip',
 		'libxml2'       => '64bit-gcc4/libxml2-2.7.3-bin_20100110.zip',
@@ -128,10 +146,16 @@ Readonly my %LIBRARIES_S => {
 		'libgd'         => '64bit-gcc4/gd-2.0.35-bin_20100110.zip',
 		'libfreetype'   => '64bit-gcc4/freetype-2.3.11-bin_20100110.zip',
 		'libopenssl'    => '64bit-gcc4/openssl-1.0.0-beta4-bin_20100110.zip',
-		'libpostgresql' => undef, # '64bit-gcc4/postgresql-8.4.1-bin_20100110.zip'
+		'libpostgresql' => '64bit-gcc4/postgresql-8.4.1-bin_20100110.zip',
 		'libdb'         => '64bit-gcc4/db-4.8.24-bin_20100110.zip',
 		'libgdbm'       => '64bit-gcc4/gdbm-1.8.3-bin_20100112.zip',
 		'libxpm'        => '64bit-gcc4/libXpm-3.5.8-bin_20100110.zip',
+		'libxz'         => '64bit-gcc4/liblzma-xz-4.999.9beta-bin_20100308.zip',
+		'mpc'           => '64bit-gcc4/mpc-0.8.1-bin_20100306.zip',
+		'mpfr'          => '64bit-gcc4/mpfr-2.4.2-bin_20100306.zip',
+		'libmysql'      => '64bit-gcc4/mysql-5.1.44-bin_20100304.zip',
+		'freeglut'      => '64bit-gcc4/freeglut-2.6.0-bin_20100213.zip',
+		'libssh2'       => '64bit-gcc4/libssh2-1.2.5-bin_20100520.zip',
 	},
 };
 
@@ -192,11 +216,13 @@ sub get_library_file_versioned {
 	return $package_file;
 }
 
+
+
 =pod
 
 =head2 install_patch
 
-  $dist->install_patch;
+  $dist->install_patch();
 
 The C<install_patch> method can be used to install a copy of the Unix
 patch program into the distribution.
@@ -213,9 +239,8 @@ sub install_patch {
 		install_to => q{.},
 		url        => $self->_binary_url($self->get_library_file('patch')),
 	);
-	$self->{bin_patch} = File::Spec->catfile(
-		$self->image_dir, 'c', 'bin', 'patch.exe',
-	);
+	$self->{bin_patch} = $self->file(qw(c bin patch.exe));
+
 	unless ( -x $self->bin_patch() ) {
 		die "Can't execute patch";
 	}
@@ -229,7 +254,7 @@ sub install_patch {
 
 =head2 bin_patch
 
-  $dist->bin_patch;
+  $dist->bin_patch();
 
 The C<bin_patch> method returns the location of the patch.exe file 
 installed by L</install_patch>.
@@ -240,6 +265,8 @@ sub bin_patch {
 	return $_[0]->{bin_patch};
 }
 
+
+
 =pod
 
 =head2 install_ppm
@@ -255,7 +282,7 @@ sub install_ppm {
 	my $self = shift;
 
 	# Where should the ppm build directory be
-	my $ppmdir = catdir( $self->image_dir(), 'ppm', );
+	my $ppmdir = $self->dir('ppm');
 	if ( -d $ppmdir ) {
 		die("PPM build directory '$ppmdir' already exists");
 	}
@@ -287,16 +314,16 @@ sub install_ppm {
 
 		# Install PPM itself
 		my $share = $self->dist_dir();
-		if ($self->portable()) {
+		if ($self->portable() && (12 < $self->perl_major_version()) ) {
 			$self->install_distribution_from_file(
 				mod_name      => 'PPM',
-				file          => catfile($share, 'modules', 'PPM-0.01_02.tar.gz'),
+				file          => catfile($share, 'modules', 'PPM-0.01_03.tar.gz'),
 				makefilepl_param => ['INSTALLDIRS=site'],
 			);		
 		} else {
 			$self->install_distribution_from_file(
 				mod_name      => 'PPM',
-				file          => catfile($share, 'modules', 'PPM-0.01_02.tar.gz'),
+				file          => catfile($share, 'modules', 'PPM-0.01_03.tar.gz'),
 				makefilepl_param => ['INSTALLDIRS=vendor'],
 			);		
 		}
@@ -310,6 +337,8 @@ sub install_ppm {
 	return 1;
 }
 
+
+
 =pod
 
 =head2 install_win32_manifest
@@ -352,48 +381,26 @@ sub install_win32_manifest {
 END_MANIFEST
 
 	# Write the manifest
-	my $file = File::Spec->catfile( $self->image_dir(), @_ );
+	my $file = $self->file(@_);
 	unless ( -f $file ) {
 		die "Program $file does not exist";
 	}
 
-	SCOPE: {
-		local *FILE;
-		open( FILE, '>', "$file.manifest" ) or die "open: $!";
-		print FILE $manifest                or die "print: $!";
-		close( FILE )                       or die "close: $!";
-	}
+	my $manifest_file;
+	open $manifest_file, '>', "$file.manifest" or die "open: $!";
+	print { $manifest_file } $manifest         or die "print: $!";
+	close $manifest_file                       or die "close: $!";
 
 	return 1;	
 }
 
-=pod
 
-=head2 install_dbd_mysql
-
-  $dist->install_dbd_mysql;
-
-Installs DBD::mysql from the PAR files on the Strawberry Perl web site.
-
-=cut
-
-sub install_dbd_mysql {
-	my $self = shift;
-
-	my $url = $self->get_library_file_versioned('mysql');
-
-	my $filelist = $self->install_par(
-	  name => 'DBD::mysql', 
-	  url => $self->_binary_url($url)
-	);
-		
-}
 
 =pod
 
-=head3 install_pari
+=head2 install_pari
 
-  $dist->install_pari
+  $dist->install_pari()
 
 The C<install_pari> method install (via a PAR package) libpari and the
 L<Math::Pari> module into the distribution.
@@ -415,331 +422,55 @@ sub install_pari {
 	return 1;
 } ## end sub install_pari
 
-=pod
-
-=head2 install_zlib
-
-  $dist->install_zlib
-
-The C<install_zlib> method installs the B<GNU zlib> compression library
-into the distribution, and is typically installed during "C toolchain"
-build phase.
-
-It provides the appropriate arguments to a C<install_library> call that
-will extract the standard zlib win32 package, and generate the additional
-files that Perl needs.
-
-Returns true or throws an exception on error.
-
-=cut
-
-sub install_zlib {
-	my $self = shift;
-
-	my $filelist = $self->install_binary(
-		name      => 'zlib',
-		install_to => q{.},
-		url       => $self->_binary_url($self->get_library_file('zlib')),
-	);
-
-	$self->insert_fragment( 'zlib', $filelist );
-
-	return 1;
-} ## end sub install_zlib
-
-=pod
-
-=head2 install_libiconv
-
-  $dist->install_libiconv
-
-The C<install_libiconv> method installs the C<GNU libiconv> library,
-which is used for various character encoding tasks, and is needed for
-other libraries such as C<libxml>.
-
-Returns true or throws an exception on error.
-
-=cut
-
-sub install_libiconv {
-	my $self     = shift;
-
-	my $filelist = $self->install_binary( 
-		name => 'libiconv', 
-		install_to => q{.},
-		url  => $self->_binary_url($self->get_library_file('libiconv')),
-	);
-
-	$self->insert_fragment( 'libiconv', $filelist );
-
-	return 1;
-} ## end sub install_libiconv
 
 
 =pod
 
-=head2 install_libxml
+=head2 install_librarypack
 
-  $dist->install_libxml
+  $dist->install_librarypack('zlib')
 
-The C<install_libxml> method installs the C<Gnome libxml> library,
-which is a fast, reliable, XML parsing library, and the new standard
-library for XML parsing.
-
-Returns true or throws an exception on error.
+The C<install_librarypack> method installs a library defined in 
+C<%Perl::Dist::Strawberry::LIBRARIES_S>.
 
 =cut
 
-sub install_libxml {
+sub install_librarypack {
 	my $self = shift;
-
+	my $library = shift;
+	
 	my $filelist = $self->install_binary(
-		name       => 'libxml2',
-		install_to => q{.},
-		url        => $self->_binary_url($self->get_library_file('libxml2')),
+		name       => $library,
+		url        => $self->_binary_url($self->get_library_file($library)),
+		install_to => q{.}
 	);
+	$self->insert_fragment($library, $filelist);
 
-	$self->insert_fragment( 'libxml', $filelist );
 
 	return 1;
-} ## end sub install_libxml
+}
+
+
 
 =pod
 
-=head2 install_expat
+=head2 install_librarypacks
 
-  $dist->install_expat
+  $dist->install_librarypacks(qw{zlib libiconv})
 
-The C<install_expat> method installs the C<Expat> XML library,
-which was the first popular C XML parser. Many Perl XML libraries
-are based on Expat.
-
-Returns true or throws an exception on error.
+The C<install_librarypacks> method installs a list of libraries defined 
+in C<%Perl::Dist::Strawberry::LIBRARIES_S>.
 
 =cut
 
-sub install_expat {
+sub install_librarypacks {
 	my $self = shift;
 
-	my $filelist = $self->install_binary(
-		name       => 'libexpat',
-		install_to => q{.},
-		url        => $self->_binary_url($self->get_library_file('libexpat')),
-	);
+	foreach my $library (@_) {
+		$self->install_librarypack( $library );
+	}
 
-	$self->insert_fragment( 'libexpat', $filelist );
-
-	return 1;
-} ## end sub install_expat
-
-=pod
-
-=head2 install_gmp
-
-  $dist->install_gmp
-
-The C<install_gmp> method installs the C<GNU Multiple Precision Arithmetic
-Library>, which is used for fast and robust bignum support.
-
-Returns true or throws an exception on error.
-
-=cut
-
-sub install_gmp {
-	my $self = shift;
-
-	# Comes as a single prepackaged vanilla-specific zip file
-	my $filelist = $self->install_binary( 
-		name => 'gmp', 
-		install_to => q{.},
-		url  => $self->_binary_url($self->get_library_file('gmp')),
-	);
-
-	$self->insert_fragment( 'gmp', $filelist );
-
-	return 1;
-}
-
-sub install_libxslt {
-	my $self = shift;
-
-	my $filelist = $self->install_binary(
-		name       => 'libxslt',
-		install_to => q{.},
-		url        => $self->_binary_url($self->get_library_file('libxslt')),
-	);
-
-	$self->insert_fragment( 'libxslt', $filelist );
-
-	return 1;
-}
-
-
-
-sub install_libjpeg {
-	my $self = shift;
-
-	my $filelist = $self->install_binary(
-		name       => 'libjpeg',
-		url        => $self->_binary_url($self->get_library_file('libjpeg')),
-		install_to => q{.}
-	);
-	$self->insert_fragment('libjpeg', $filelist);
-
-	return 1;
-}
-
-
-sub install_libgif {
-	my $self = shift;
-
-	my $filelist = $self->install_binary(
-		name       => 'libgif',
-		url        => $self->_binary_url($self->get_library_file('libgif')),
-		install_to => q{.}
-	);
-	$self->insert_fragment('libgif', $filelist);
-
-	return 1;
-}
-
-
-sub install_libpng {
-	my $self = shift;
-
-	my $filelist = $self->install_binary(
-		name       => 'libpng',
-		url        => $self->_binary_url($self->get_library_file('libpng')),
-		install_to => q{.}
-	);
-	$self->insert_fragment('libpng', $filelist);
-
-
-	return 1;
-}
-
-
-sub install_libtiff {
-	my $self = shift;
-
-	my $filelist = $self->install_binary(
-		name       => 'libtiff',
-		url        => $self->_binary_url($self->get_library_file('libtiff')),
-		install_to => q{.}
-	);
-	$self->insert_fragment('libtiff', $filelist);
-
-
-	return 1;
-}
-
-
-sub install_libgd {
-	my $self = shift;
-
-	my $filelist = $self->install_binary(
-		name       => 'libgd',
-		url        => $self->_binary_url($self->get_library_file('libgd')),
-		install_to => q{.}
-	);
-	$self->insert_fragment('libgd', $filelist);
-
-
-	return 1;
-}
-
-sub install_libfreetype {
-	my $self = shift;
-
-	my $filelist = $self->install_binary(
-		name       => 'libfreetype',
-		url        => $self->_binary_url($self->get_library_file('libfreetype')),
-		install_to => q{.}
-	);
-	$self->insert_fragment('libfreetype', $filelist);
-
-	return 1;
-}
-
-sub install_libopenssl {
-	my $self = shift;
-
-	my $filelist = $self->install_binary(
-		name       => 'libopenssl',
-		url        => $self->_binary_url($self->get_library_file('libopenssl')),
-		install_to => q{.}
-	);
-	$self->insert_fragment('libopenssl', $filelist);
-
-
-	return 1;
-}
-
-sub install_libpostgresql {
-	my $self = shift;
-
-	my $filelist = $self->install_binary(
-		name       => 'libpostgresql',
-		url        => $self->_binary_url($self->get_library_file('libpostgresql')),
-		install_to => q{.}
-	);
-	$self->insert_fragment('libpostgresql', $filelist);
-
-	return 1;
-}
-
-=pod
-
-=head2 install_libdb
-
-  $dist->install_libdb;
-
-The C<install_libdb> method can be used to install a copy of the 
-Berkeley DB library.
-
-Returns true or throws an exception on error.
-
-=cut
-
-
-sub install_libdb {
-	my $self = shift;
-
-	my $filelist = $self->install_binary(
-		name       => 'libdb',
-		url        => $self->_binary_url($self->get_library_file('libdb')),
-		install_to => q{.}
-	);
-	$self->insert_fragment('libdb', $filelist);
-
-	return 1;
-}
-
-sub install_libgdbm {
-	my $self = shift;
-
-	my $filelist = $self->install_binary(
-		name       => 'libgdbm',
-		url        => $self->_binary_url($self->get_library_file('libgdbm')),
-		install_to => q{.}
-	);
-	$self->insert_fragment('libgdbm', $filelist);
-
-	return 1;
-}
-
-sub install_libxpm {
-	my $self = shift;
-
-	my $filelist = $self->install_binary(
-		name       => 'libxpm',
-		url        => $self->_binary_url($self->get_library_file('libxpm')),
-		install_to => q{.}
-	);
-	$self->insert_fragment('libxpm', $filelist);
-
-
-	return 1;
+	return $1;
 }
 
 1;
